package com.bregant.azure_speech_recognition

import androidx.annotation.NonNull;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import com.microsoft.cognitiveservices.speech.ResultReason;
import com.microsoft.cognitiveservices.speech.SpeechConfig;
import com.microsoft.cognitiveservices.speech.SpeechRecognitionResult;
import com.microsoft.cognitiveservices.speech.SpeechRecognizer;
import java.util.concurrent.Future;
//import androidx.core.app.ActivityCompat;
import java.net.URI;
import android.util.Log;

/** AzureSpeechRecognitionPlugin */
public class AzureSpeechRecognitionPlugin: FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "azure_speech_recognition")
    channel.setMethodCallHandler(this);
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "azure_speech_recognition")
      channel.setMethodCallHandler(AzureSpeechRecognitionPlugin())
    }
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
      if(call.method == "simpleVoice"){
        //_result = result;
        var permissionRequestId : Int = 5;
        var speechSubscriptionKey : String = "" + call.argument("subscriptionKey");
        var serviceRegion : String= ""+call.argument("region");
        var lang : String = ""+call.argument("language");


        var text = simpleSpeechRecognition(speechSubscriptionKey,serviceRegion,lang);
        result.success(text);

    }else{
      result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  fun simpleSpeechRecognition(speechSubscriptionKey:String,serviceRegion:String,lang:String) : String{
    try{
      

      var config : SpeechConfig = SpeechConfig.fromSubscription(speechSubscriptionKey, serviceRegion); 
      assert(config != null);

      config.speechRecognitionLanguage = lang;

      var reco : SpeechRecognizer = SpeechRecognizer(config);

      assert(reco != null);

      var task : Future<SpeechRecognitionResult> = reco.recognizeOnceAsync();

      assert(task != null);

      var response : SpeechRecognitionResult = task.get();

      assert(response != null);

      if(response.getReason() == ResultReason.RecognizedSpeech){
        println(response.toString());
        return response.toString();
        
      }else{
        return "Error recognizing. Did you update the subscription info?" + System.lineSeparator() + response.toString()
      }
      reco.close();

    }catch(exec:Exception){
      Log.e("SpeechSDKDemo", "unexpected " + exec.message);
      assert(false);
      return "unexpected " + exec.message;
    }
  }
}
